/* SoundcloudLibrary.h */

/* Copyright (C) 2014  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef SOUNDCLOUD_H
#define SOUNDCLOUD_H

#include "SoundcloudData.h"
#include "Components/Library/AbstractLibrary.h"


class SoundcloudLibrary : public AbstractLibrary
{
	Q_OBJECT

signals:
	// called, when webservice returns artists/albums/tracks
	void sig_artists_found(const ArtistList& artists);
	void sig_albums_found(const AlbumList& albums);
	void sig_tracks_found(const MetaDataList& v_md);

public:
	SoundcloudLibrary(QObject *parent=nullptr);
	virtual ~SoundcloudLibrary();

	virtual void load();
	void				search_artist(const QString& artist_name);
	void				fetch_tracks_by_artist(qint64 artist_sc_id);
	void				fetch_playlists_by_artist(qint64 artist_sc_id);
	virtual void		insert_tracks(const MetaDataList& v_md);
	virtual void		insert_tracks(const MetaDataList& v_md, const ArtistList& artists, const AlbumList& albums);

protected:

	SoundcloudData*		_scd;

	virtual void		get_all_artists(ArtistList& artists, LibSortOrder so=LibSortOrder()) override;
	virtual void		get_all_artists_by_searchstring(Filter filter, ArtistList& artists, LibSortOrder so) override;

	virtual void		get_all_albums(AlbumList& albums, LibSortOrder so) override;
	virtual void		get_all_albums_by_artist(QList<int> artist_ids, AlbumList& albums, Filter filter, LibSortOrder so) override;
	virtual void		get_all_albums_by_searchstring(Filter filter, AlbumList& albums, LibSortOrder so) override;

	virtual void		get_all_tracks(const QStringList& paths, MetaDataList& v_md) override;
	virtual void		get_all_tracks(MetaDataList& v_md, LibSortOrder so) override;
	virtual void		get_all_tracks_by_artist(QList<int> artist_ids, MetaDataList& v_md, Filter filter, LibSortOrder so) override;
	virtual	void		get_all_tracks_by_album(QList<int> album_ids, MetaDataList& v_md, Filter filter, LibSortOrder so) override;
	virtual void		get_all_tracks_by_searchstring(Filter filter, MetaDataList& v_md, LibSortOrder so) override;

	virtual void		update_track(const MetaData& md);
	virtual void		update_album(const Album& album);

	virtual void		delete_tracks(const MetaDataList& v_md, TrackDeletionMode mode);
	virtual void		get_album_by_id(int album_id, Album& album);
	virtual void		get_artist_by_id(int artist_id, Artist& artist);

	MetaDataList		_fetched_tracks;
	AlbumList			_fetched_albums;
	ArtistList			_fetched_artists;

	void				apply_artist_and_album_to_md();


private slots:
	virtual void		artists_fetched(const ArtistList& artists);
	virtual void		tracks_fetched(const MetaDataList& v_md);
	virtual void		albums_fetched(const AlbumList& albums);

	void				cover_lookup_finished(bool);
	void				cover_found(const CoverLocation& cl);

public slots:
	virtual void		psl_reload_library(bool);
	virtual void		refresh_artist();
	virtual void		refresh_albums();
	virtual void		refresh_tracks();
};

#endif // LocalLibrary_H
