/* LN_Notification.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <libnotify/notify.h>

#include <QFile>
#include <QPixmap>

#include "Components/CoverLookup/CoverLocation.h"
#include "Interfaces/NotificationInterface/Notification.h"
#include "LN_Notification.h"


LN_Notification::LN_Notification(QObject* parent) :
	Notification(parent)
{
	_initialized = notify_init("Sayonara"); 
    _not = 0;
}

LN_Notification::~LN_Notification() {

}

static void notification_closed(gpointer notification, void* data){

	Q_UNUSED(data)

    gpointer* pnot = &notification;
    *pnot = 0;

}

void LN_Notification::notification_show(const QString& title, const QString& label){
	if(!_initialized) {
		return;
	}

	QString pixmap_path;

	not_close();

	pixmap_path = Helper::getSharePath() + "logo.png";

	NotifyNotification* n = notify_notification_new( title.toLocal8Bit().data(),
												 label.toLocal8Bit().data(),
												pixmap_path.toLocal8Bit().data());

	 g_signal_connect(n, "closed", G_CALLBACK(notification_closed), nullptr);

   _not = n;

	int timeout = _settings->get(Set::Notification_Timeout);

	notify_notification_set_timeout     (n, timeout);
	notify_notification_show            (n, nullptr);
}

void LN_Notification::notification_show(const QString& str){
	notification_show("Sayonara", str);
}

void LN_Notification::notification_show(const MetaData& md) {

	if(!_initialized) {
		return;
	}

	CoverLocation cl = CoverLocation::get_cover_location(md);

    QString pixmap_path;

    not_close();

    QString text = md.artist + "\n" + md.album;
    text.replace("&", "&amp;");

	if( !QFile::exists(cl.cover_path) ) {
		pixmap_path = Helper::getSharePath() + "logo.png";
	}

    else{

		bool success;
        QPixmap p(cl.cover_path);

		int scale = _settings->get(Set::Notification_Scale);

		if(scale > 0) {
			p = p.scaled(scale, scale, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
		}

		success = p.save(Helper::getSayonaraPath() + "not.jpg");

		if(success) {
            pixmap_path = Helper::getSayonaraPath() + "not.jpg";
		}
    }


	char* not_title = md.title.toLocal8Bit().data();
	char* not_text = text.toLocal8Bit().data();
	char* not_pixmap_data = pixmap_path.toLocal8Bit().data();

	NotifyNotification* n = notify_notification_new( not_title, not_text, not_pixmap_data);

     g_signal_connect(n, "closed", G_CALLBACK(notification_closed), nullptr);


   _not = n;

	int timeout = _settings->get(Set::Notification_Timeout);

    notify_notification_set_timeout     (n, timeout);
    notify_notification_show            (n, nullptr);

}

void LN_Notification::notification_update(const MetaData& md) {

   if(!_not) return;

   QString pixmap_path = CoverLocation::get_cover_location(md).cover_path;

   QString text = md.artist + "\n" + md.album;
   text.replace("&", "&amp;");

   notify_notification_update( (NotifyNotification*) _not, 
				md.title.toLocal8Bit().data(),
				text.toLocal8Bit().data(),
				pixmap_path.toLocal8Bit().data());
}

void LN_Notification::not_close() {

    if(!_not) return;

    NotifyNotification* n = (NotifyNotification*) _not;
    if(n){
       notify_notification_close(n,nullptr);
    }

    _not = nullptr;
}


QString LN_Notification::get_name() {
	return "libnotify";
}

