/* application.h */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef APPLICATION_H
#define APPLICATION_H

#define CONNECT(a,b,c,d) app->connect(a, SIGNAL(b), c, SLOT(d))

#include <QApplication>
#include <QTranslator>
#include <QFont>

#include "Components/DBus/DBusHandler.h"
#include "Components/RadioStation/StreamServer.h"
#include "Components/Playlist/PlaylistHandler.h"

#include "Interfaces/LibraryInterface/LibraryPluginLoader.h"
#include "Interfaces/PlayerPlugin/PlayerPluginHandler.h"

#include "GUI/Player/GUI_Player.h"
#include "GUI/Playlist/GUI_Playlist.h"

#include "GUI/PlayerPlugins/PlaylistChooser/GUI_PlaylistChooser.h"
#include "GUI/PlayerPlugins/AudioConverter/GUI_AudioConverter.h"
#include "GUI/PlayerPlugins/Engine/GUI_LevelPainter.h"
#include "GUI/PlayerPlugins/Engine/GUI_Spectrum.h"
#include "GUI/PlayerPlugins/Stream/GUI_Stream.h"
#include "GUI/PlayerPlugins/Stream/GUI_Podcasts.h"
#include "GUI/PlayerPlugins/Equalizer/GUI_Equalizer.h"
#include "GUI/PlayerPlugins/Bookmarks/GUI_Bookmarks.h"
#include "GUI/PlayerPlugins/Speed/GUI_Speed.h"
#include "GUI/PlayerPlugins/Broadcasting/GUI_Broadcast.h"

class Application : public QApplication, private SayonaraClass
{
    Q_OBJECT

public:
	Application(int& argc, char** argv);
    virtual ~Application();

	bool init(QTranslator* translator, const QStringList& files_to_play);

private:
	GUI_Player*             player;

	PlayerPluginHandler*    _pph;

	GUI_PlaylistChooser*    ui_playlist_chooser;
	GUI_AudioConverter*     ui_audioconverter;
	GUI_LevelPainter*       ui_level;
	GUI_Spectrum*           ui_spectrum;
	GUI_Stream*				ui_stream;
	GUI_Podcasts*           ui_podcasts;
	GUI_Equalizer*          ui_eq;
	GUI_Bookmarks*          ui_bookmarks;
	GUI_Speed*				ui_speed;
	GUI_Broadcast*			ui_broadcast;

	GUI_Playlist*           ui_playlist;
	StreamServer*           stream_server;

	DBusHandler*			_dbus;

	void check_for_crash();
};

#endif // APPLICATION_H


