/* Bookmarks.h */

/* Copyright (C) 2011-2014  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef BOOKMARKS_H
#define BOOKMARKS_H

#include "Components/PlayManager/PlayManager.h"
#include "DatabaseAccess/CDatabaseConnector.h"

#include <QPair>
#include <QList>

class Bookmark{

private:
	quint32 _time;
	QString _name;

public:
	Bookmark(quint32 time=0, const QString& name=QString()){
		_time = time;
		_name = name;
	}

	quint32 get_time() const {
		return _time;
	}

	QString get_name() const {
		return _name;
	}


};


class Bookmarks : public QObject
{
	Q_OBJECT

signals:
	void sig_bookmarks_changed(const QList<Bookmark>& bookmarks);

	void sig_prev_changed(const Bookmark&, bool valid);
	void sig_next_changed(const Bookmark&, bool valid);



public:
	explicit Bookmarks(QObject *parent=nullptr);

	bool is_md_extern();
	bool save();

	bool jump_to(int idx);
	bool jump_next();
	bool jump_prev();

	bool remove(int idx);
	bool remove_all();

	bool set_loop(bool);
	QList<Bookmark> get_all_bookmarks();


private slots:
	void pos_changed_ms(quint64 pos);
	void track_changed(const MetaData& md);
	void playstate_changed(PlayManager::PlayState state);


private:
	CDatabaseConnector*		_db;
	QList<Bookmark>			_bookmarks;
	PlayManager*			_play_manager;
	MetaData				_md;

	quint32					_cur_time;
	int						_prev_idx;
	int						_next_idx;

	quint32					_loop_start;
	quint32					_loop_end;


	void reload_bookmarks();
	void sort_bookmarks();

};

#endif // BOOKMARKS_H
