/* CoverFetchThread.cpp */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * CoverFetchThread.cpp
 *
 *  Created on: Jun 28, 2011
 *      Author: luke
 */

#include "CoverFetchThread.h"

#include "Helper/Logger/Logger.h"
#include "Helper/WebAccess/AsyncWebAccess.h"

#include <QFile>
#include <QDir>

CoverFetchThread::CoverFetchThread(QObject* parent, const CoverLocation& cl, const int n_covers) :
	QObject(parent)
{

	_adresses.clear();
	_covers_found = 0;
	_n_covers = n_covers;
	_url = cl.search_url;

	_target_file = cl.cover_path;
}


CoverFetchThread::~CoverFetchThread() {

}

bool CoverFetchThread::start(){

	if(!_url.contains("google", Qt::CaseInsensitive))

	{
		_adresses << _url;
		more();
		return true;
	}

	AsyncWebAccess* awa = new AsyncWebAccess(this);
	connect(awa, SIGNAL(sig_finished(bool)), this, SLOT(content_fetched(bool)));
	awa->run(_url);
	return true;

}


bool CoverFetchThread::more(){

	if(_n_covers == _covers_found || _adresses.isEmpty()){
		emit sig_finished(true);
		return false;
	}

	QString adress = _adresses[0];
	AsyncWebAccess* awa = new AsyncWebAccess(this);

	if(_n_covers == 1){
		connect(awa, SIGNAL(sig_finished(bool)), this, SLOT(single_image_fetched(bool)));
	}

	else{
		connect(awa, SIGNAL(sig_finished(bool)), this, SLOT(multi_image_fetched(bool)));
	}


	awa->run(adress);
	_adresses.pop_front();

	return true;
}




void CoverFetchThread::content_fetched(bool success){

	if(!success){
		sp_log(Log::Warning) << "Could not fetch content";
		return;
	}


	AsyncWebAccess* awa = (AsyncWebAccess*) sender();
	QByteArray website = awa->get_data();

	if(_n_covers == 1){

		_adresses = calc_adresses_from_google(10, website);

	}

	else{
		_adresses = calc_adresses_from_google(10, website);
	}

	if(_adresses.isEmpty()){
		emit sig_finished(false);
		return;
	}

	more();
}


void CoverFetchThread::single_image_fetched(bool success){

	if(!success){
		sp_log(Log::Warning) << "Could not fetch cover";
		return;
	}

	AsyncWebAccess* awa = (AsyncWebAccess*) sender();
	QImage img  = awa->get_image();

	if(!img.isNull()){

		_covers_found++;
		save_and_emit_image(_target_file, img, awa->get_url());

	}
}


void CoverFetchThread::multi_image_fetched(bool success){

	if(!success){
		sp_log(Log::Warning) << "Could not fetch cover";
		return;
	}

	AsyncWebAccess* awa = (AsyncWebAccess*) sender();
	QImage img  = awa->get_image();

	if(!img.isNull()){
		QString filename, dir;
		Helper::split_filename(_target_file, dir, filename);

		QString cover_path = dir + "/" + QString::number(_covers_found) + "_" + filename;

		save_and_emit_image(cover_path, img, awa->get_url());

		_covers_found++;
	}
}



void CoverFetchThread::save_and_emit_image(const QString& filepath, const QImage& img, const QString& url){

	bool success = img.save(filepath);
	if(!success){
		sp_log(Log::Warning) << "Cannot save image to " << filepath;
	}

	CoverLocation cl;
	cl.cover_path = filepath;
	cl.search_url = url;
	cl.valid = true;

	emit sig_cover_found(cl);
}


QStringList CoverFetchThread::calc_adresses(int num, const QByteArray& website, const QString& regex){

	QStringList adresses;
	if (website.isEmpty()) {
		sp_log(Log::Error) << "Cover Fetch Thread: website empty";
		return adresses;
	}

	int n_covers = 0;
	int idx=50000;


	QString website_str = QString::fromLocal8Bit(website);

	while(n_covers < num) {
		QRegExp re(regex);
		re.setMinimal(true);
		idx = re.indexIn(website_str, idx);

		if(idx == -1) break;

		QString str = re.cap(0);

		idx += str.length();
		str.remove("\"");
		adresses << str;
		n_covers++;
	}

	sp_log(Log::Debug) << "Got " << adresses.size() << " adresses";
	return adresses;

}

QStringList CoverFetchThread::calc_adresses_from_google(int num, const QByteArray& website) {
	return calc_adresses(num, website, QString("(https://encrypted-tbn)(\\S)+(\")"));
}
