/* CoverFetchThread.h */

/* Copyright (C) 2011  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


/*
 * CoverFetchThread.h
 *
 *  Created on: Jun 28, 2011
 *      Author: luke
 */

#ifndef COVERFETCHTHREAD_H_
#define COVERFETCHTHREAD_H_

#include "Helper/Helper.h"
#include "Helper/WebAccess/AsyncWebAccess.h"
#include "Components/CoverLookup/CoverLocation.h"

#include <QImage>

class CoverFetchThread : public QObject {

Q_OBJECT

signals:
    void sig_finished(bool);
	void sig_cover_found(const CoverLocation&);

public:

	CoverFetchThread(QObject* parent, const CoverLocation& cl, const int n_covers);
    virtual ~CoverFetchThread();

	bool start();
	bool more();


private:

	AsyncWebAccess*		_single_img_fetcher;
	AsyncWebAccess*		_multi_img_fetcher;
	AsyncWebAccess*		_content_fetcher;

	QString				_url;
	QString				_target_file;
	QStringList			_adresses;

	int					_n_covers;
	int					_covers_found;



	QStringList calc_adresses(int num, const QByteArray& website, const QString& regex);
	QStringList calc_adresses_from_google(int num, const QByteArray& website);
	void save_and_emit_image(const QString& filepath, const QImage& img, const QString& url);


private slots:
	void single_image_fetched(bool success);
	void multi_image_fetched(bool success);
	void content_fetched(bool success);

};

#endif /* COVERFETCHTHREAD_H_ */
