/* Engine.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "Engine.h"


Engine::Engine(QObject *parent) :
	QObject(parent),
	SayonaraClass()
{
	_uri = nullptr;
	_cur_pos_ms = 0;

	REGISTER_LISTENER(Set::Engine_ShowLevel, _sl_show_level_changed);
	REGISTER_LISTENER(Set::Engine_ShowSpectrum, _sl_show_spectrum_changed);
	REGISTER_LISTENER(Set::BroadCast_Active, _sl_broadcast_active_changed);
}

QString Engine::get_name() const {
	return _name;
}

void Engine::set_track_finished(){

}

void Engine::set_level(float right, float left){
	emit sig_level(right, left);
}

void Engine::set_spectrum(QList<float> &lst){
	emit sig_spectrum(lst);
}

void Engine::async_done(){

}

void Engine::update_md(const MetaData& md){
	Q_UNUSED(md)
}

void Engine::update_duration(){

}

void Engine::update_time(qint32 time){
	Q_UNUSED(time)
}

void Engine::update_bitrate(quint32 br){
	Q_UNUSED(br)
}

void Engine::unmute(){

}

bool Engine::get_show_level() const {
	return _show_level;
}

bool Engine::get_show_spectrum() const {
	return _show_spectrum;
}

bool Engine::get_broadcast_active() const {
	return _broadcast_active;
}

void Engine::set_track_ready(){

}

void Engine::sr_initialized(bool b){
	if(b) {
		play();
	}
}

void Engine::sr_not_valid(){
	emit sig_track_finished();
}

void Engine::set_about_to_finish(qint64 ms){
	Q_UNUSED(ms)
}

void Engine::set_cur_position_ms(qint64 ms){
	_cur_pos_ms = ms;
	emit sig_pos_changed_ms(ms);
}

void Engine::_sl_show_level_changed(){
	_show_level = _settings->get(Set::Engine_ShowLevel);
}


void Engine::_sl_show_spectrum_changed(){
	_show_spectrum = _settings->get(Set::Engine_ShowSpectrum);
}

void Engine::_sl_broadcast_active_changed(){
	_broadcast_active = _settings->get(Set::BroadCast_Active);
}


void Engine::new_data(uchar* data, quint64 size){
	emit sig_data(data, size);
}

void Engine::buffering(int progress)
{
	Q_UNUSED(progress);
}

void Engine::eq_changed(int band, int value){
	Q_UNUSED(band)
	Q_UNUSED(value)
}

void Engine::record_button_toggled(bool b){
	_sr_wanna_record = b;
}

void Engine::psl_new_stream_session(){

}

void Engine::start_convert(){

}

void Engine::end_convert(){

}

void Engine::psl_set_speed(float f){
	Q_UNUSED(f);
}

