/* GSTEngineHandler.h */

/* Copyright (C) 2014  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ENGINEHANDLER_H_
#define ENGINEHANDLER_H_

#include "Components/Engine/Engine.h"
#include "Components/PlayManager/PlayManager.h"

#include <QVector>

class GSTEngineHandler : public Engine
{
	Q_OBJECT
	SINGLETON_QOBJECT(GSTEngineHandler)

public:

	void fill_engines(const QVector<Engine*>& engines);
	virtual bool init();


public slots:

	void jump_abs_ms(quint64 ms) override;
	void jump_rel_ms(quint64 ms) override;
	void jump_rel(double where) override;

	void change_track(const MetaData&) override;
	void change_track(const QString&) override;

	// TODO: Why override?
	void eq_changed(int band, int value) override;
	void record_button_toggled(bool) override;

	void psl_new_stream_session() override;

    void psl_change_engine(QString name);

	// TODO: Why override?
	void start_convert() override;
	void end_convert() override;

	void psl_set_speed(float f) override;
	void set_track_ready() override;
	void buffering(int progress) override;


private slots:

	void sl_md_changed(const MetaData&);
	void sl_dur_changed(const MetaData&);
	void sl_pos_changed_ms(quint64);
	void sl_pos_changed_s(quint32);

	void sl_track_finished();
	void sl_level(float, float);
	void sl_spectrum(const QList<float>&);



protected slots:

	void playstate_changed(PlayManager::PlayState);

	void play() override;
	void stop() override;
	void pause() override;



private:

	PlayManager*		_play_manager;
	Engine*				_cur_engine;
	QVector<Engine*>	_engines;

	bool configure_connections(Engine* old_engine, Engine* new_engine);
};

#endif


