/* GSTPlaybackEngine.h */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef GSTPLAYBACKENGINE_H_
#define GSTPLAYBACKENGINE_H_


#define EQ_TYPE_NONE -1
#define EQ_TYPE_KEQ 0
#define EQ_TYPE_10B 1

#include "GSTPlaybackPipeline.h"
#include "StreamRecorder.h"
#include "Engine.h"

#include "Helper/Equalizer_presets.h"

#include <glib.h>
#include <gst/gst.h>
#include <gst/gstbuffer.h>


enum class GaplessState : quint8 {
	NoGapless=0,		// no gapless enabled at all
	AboutToFinish,		// the phase when the new track is already displayed but not played yet
	TrackFetched,		// track is requested, but no yet there
	Playing				// currently playing
};

class GSTPlaybackEngine : public Engine {

	Q_OBJECT

public:

	GSTPlaybackEngine(QObject* parent=nullptr);
	virtual ~GSTPlaybackEngine();


	void        emit_buffer(float inv_array_elements, float scale);
	void		set_track_finished() override;
	void		update_duration() override;
	void		update_bitrate(quint32 br) override;

	void		set_about_to_finish(qint64 time2go) override;
	void		set_cur_position_ms(qint64 pos_ms) override;


	MyCaps*		get_caps() const;
	void		unmute();

	virtual bool init();


private:
	
	GSTPlaybackPipeline*	_pipeline;
	GSTPlaybackPipeline*	_other_pipeline;

	StreamRecorder*			_stream_recorder;

	MyCaps*					_caps;
	MetaData				_md_gapless;

	GaplessState			_gapless_state;

	bool					_sr_active;
	int						_jump_play_s;

	// methods
	bool set_uri(const MetaData& md);
	void change_track_gapless(const MetaData& md);

public slots:
	virtual void play() override;
	virtual void stop() override;
	virtual void pause() override;


	virtual void jump_abs_ms(quint64 pos_ms) override;
	virtual void jump_rel_ms(quint64 pos_ms) override;
	virtual void jump_rel(double percent) override;


	virtual void update_md(const MetaData&) override;
	virtual void change_track(const MetaData&) override;
	virtual void change_track(const QString&) override;

	// TODO: Why override?
	virtual void eq_changed(int, int) override;
	virtual void psl_set_speed(float f) override;
	virtual void record_button_toggled(bool b) override;

	virtual void set_track_ready() override;
	virtual void buffering(int progress) override;

private slots:
	void _change_sr_active();
	void _change_gapless();

};

#endif /* GSTENGINE_H_ */
