/* DatabaseLibrary.cpp */

/* Copyright (C) 2011-2014  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "DatabaseAccess/DatabaseLibrary.h"
#include "DatabaseAccess/DatabaseAlbums.h"
#include "DatabaseAccess/DatabaseArtists.h"
#include "DatabaseAccess/DatabaseTracks.h"

DatabaseLibrary::DatabaseLibrary(QSqlDatabase db, quint8 db_id) :
	DatabaseModule(db, db_id)
{
}



void DatabaseLibrary::deleteTracksAlbumsArtists() {

	DB_RETURN_NOT_OPEN_VOID(_db);

	_db.transaction();

	bool err = false;

	for(int i=0; i<3; i++) {
		SayonaraQuery q (_db);

		if(i==0) q.prepare("delete from tracks;");
		else if(i==1) q.prepare("delete from artists;");
		else if(i==2) q.prepare("delete from albums;");

		if( !q.exec() ){
			q.show_error("Cannot delete all tracks, albums and artists");
			err = true;
		}
	}

	if(err){
		_db.commit();
	}
	else {
		_db.rollback();
	}
}



bool DatabaseLibrary::storeMetadata(const MetaDataList& v_md)  {

	DB_RETURN_NOT_OPEN_BOOL(_db);

	if(v_md.isEmpty()) return true;

	int artistID = -1, albumID = -1;

	_db.transaction();

	DatabaseAlbums db_albums(_db, v_md[0].db_id);
	DatabaseArtists db_artists(_db, v_md[0].db_id);
	DatabaseTracks db_tracks(_db, v_md[0].db_id);

	for(const MetaData& md : v_md) {

		//first check if we know the artist and its id

		albumID = db_albums.getAlbumID(md.album);
		if (albumID == -1) {
			albumID = db_albums.insertAlbumIntoDatabase(md.album);
		}

		artistID = db_artists.getArtistID(md.artist);
		if (artistID == -1) {
			artistID = db_artists.insertArtistIntoDatabase(md.artist);
		}

		if(albumID == -1 || artistID == -1){
			sp_log(Log::Warning) << "Cannot insert artist or album of " << md.filepath();
			continue;
		}

		db_tracks.insertTrackIntoDatabase (md, artistID, albumID);
	}

	return _db.commit();
}

