/* GUI_PlayerConnections.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "GUI_Player.h"
#include "Components/TagEdit/MetaDataChangeNotifier.h"

void GUI_Player::setup_connections() {

	qRegisterMetaType<CoverLocation>("CoverLocation");

	connect(btn_play,	SIGNAL(clicked()),	this, SLOT(play_clicked()));
	connect(btn_fw,		SIGNAL(clicked()),	this, SLOT(next_clicked()));
	connect(btn_bw,		SIGNAL(clicked()),	this, SLOT(prev_clicked()));
	connect(btn_stop,	SIGNAL(clicked()),	this, SLOT(stop_clicked()));
	connect(btn_mute,	SIGNAL(released()),	this, SLOT(mute_button_clicked()));
    connect(btn_rec,	SIGNAL(toggled(bool)), this, SLOT(rec_clicked(bool)));
	connect(albumCover, SIGNAL(clicked()), this, SLOT(cover_clicked()));

	connect(_play_manager, SIGNAL(sig_playstate_changed(PlayManager::PlayState)),
			this,			SLOT(playstate_changed(PlayManager::PlayState)));

	connect(_play_manager, SIGNAL(sig_track_changed(const MetaData&)),
			this,			SLOT(track_changed(const MetaData&)));

	connect(_play_manager, SIGNAL(sig_position_changed_ms(quint64)),
			this,			SLOT(set_cur_pos_ms(quint64)));

	connect(_play_manager, SIGNAL(sig_buffer(int)),
			this,			SLOT(buffering(int)));

	// engine
	connect(_engine, SIGNAL(sig_md_changed(const MetaData&)),
			this, SLOT(md_changed(const MetaData&)));

	connect(_engine, SIGNAL(sig_dur_changed(const MetaData&)),
			this, SLOT(dur_changed(const MetaData&)));

	connect(_engine, SIGNAL(sig_br_changed(const MetaData&)),
			this, SLOT(br_changed(const MetaData&)));

	// file
	connect(action_OpenFile, SIGNAL(triggered()),
			this, SLOT(open_files_clicked()));
	connect(action_OpenFolder, SIGNAL(triggered()),
			this, SLOT(open_dir_clicked()));
	connect(action_ImportFolder, SIGNAL(triggered()),
			this, SLOT(import_dir_clicked()));
	connect(action_ImportFiles, SIGNAL(triggered()),
			this, SLOT(import_files_clicked()));
	connect(action_reloadLibrary, SIGNAL(triggered()),
			this, SLOT(reload_library_clicked()));

	connect(action_Close, SIGNAL(triggered()), this,
				SLOT(really_close()));


	// view
	connect(action_viewLibrary, SIGNAL(toggled(bool)),
			this, SLOT(show_library(bool)));
	connect(action_Dark, SIGNAL(toggled(bool)),
			this, SLOT(skin_toggled(bool)));

	connect(action_smallPlaylistItems, SIGNAL(toggled(bool)),
			this, SLOT(small_playlist_items_toggled(bool)));
	connect(action_showOnlyTracks, SIGNAL(toggled(bool)),
			this, SLOT(only_show_tracks_toggled(bool)));
	connect(action_Fullscreen, SIGNAL(toggled(bool)),
			this, SLOT(show_fullscreen_toggled(bool)));


	// preferencesF
	connect(action_Language, SIGNAL(triggered()),
			this, SLOT(show_language_chooser()));
	connect(action_lastFM, SIGNAL(triggered()),
			this, SLOT(setup_lastfm_clicked()));
	connect(action_setLibPath, SIGNAL(triggered()),
			this, SLOT(set_library_path_clicked()));
	connect(action_startup, SIGNAL(triggered()),
			ui_StartupDialog, SLOT(show()));
	connect(action_min2tray, SIGNAL(toggled(bool)),
			this, SLOT(min2tray_toggled(bool)));
	connect(action_only_one_instance, SIGNAL(toggled(bool)),
			this, SLOT(only_one_instance_toggled(bool)));

	connect(action_streamrecorder, SIGNAL(triggered(bool)),
			this, SLOT(setup_streamrecorder_clicked()));

	connect(action_broadcasting, SIGNAL(triggered()), this,
			SLOT(setup_broadcasting_clicked()));

	connect(action_notifications, SIGNAL(triggered(bool)), ui_notifications,
            SLOT(show()));

	connect(action_livesearch, SIGNAL(triggered(bool)), this,
			SLOT(live_search_toggled(bool)));
	connect(action_notifyNewVersion, SIGNAL(triggered(bool)), this,
			SLOT(notify_new_version_toggled(bool)));


	// about
	connect(action_about, SIGNAL(triggered()), this, SLOT(about()));

	connect(action_help, SIGNAL(triggered()), this, SLOT(help()));
	connect(_tray_icon, SIGNAL(onVolumeChangedByWheel(int)),
			this, SLOT(change_volume_by_tick(int)));


	connect(volumeSlider, SIGNAL(sig_slider_moved(int)), this,
			SLOT(volume_changed(int)));

	connect(sli_progress, SIGNAL(sig_slider_moved(int)), this,
			SLOT(seek(int)));


	MetaDataChangeNotifier* md_change_notifier = MetaDataChangeNotifier::getInstance();
	connect(md_change_notifier, SIGNAL(sig_metadata_changed(const MetaDataList&, const MetaDataList&)),
			this,				SLOT(id3_tags_changed(const MetaDataList&, const MetaDataList&)));


	// cover lookup
	connect(_cov_lookup, SIGNAL(sig_cover_found(const CoverLocation&)),
			this, SLOT(set_cover_image(const CoverLocation&)));

	connect(_ui_alternative_covers, SIGNAL(sig_cover_changed(const CoverLocation&)),
			this, SLOT(set_cover_image(const CoverLocation&)));


	connect(_awa_version, SIGNAL(sig_finished(bool)), this, SLOT(awa_version_finished(bool)));
	connect(_awa_translators, SIGNAL(sig_finished(bool)), this, SLOT(awa_translators_finished(bool)));

	connect(ui_libpath, SIGNAL(sig_library_path_set()), this, SLOT(set_library_path_clicked()));

    QList<QKeySequence> lst;
	lst << QKeySequence(Qt::Key_Space);

	QAction* play_pause_action = create_actions(lst);
	connect(play_pause_action, SIGNAL(triggered()), btn_play, SLOT(click()));

	sp_log(Log::Debug) << "connections done";
}

