/* GUI_PlayerEvents.cpp */

/* Copyright (C) 2011-2014  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "GUI_Player.h"


void GUI_Player::moveEvent(QMoveEvent *e) {

	QMainWindow::moveEvent(e);

	QPoint p= this->pos();
	_settings->set(Set::Player_Pos, p);
}


void GUI_Player::resizeEvent(QResizeEvent* e) {

	QMainWindow::resizeEvent(e);

	ui_playlist->resize(playlist_widget->size());

	if(library_widget->isVisible()) {

		if( ui_libpath->isVisible() ){
			ui_libpath->resize(library_widget->size());
		}

		else{
			_cur_library->resize(library_widget->size());
		}
	}

	QSize sz = plugin_widget->size();
	QSize target_size = this->size();

	bool maximized = _settings->get(Set::Player_Maximized);
	bool fullscreen = _settings->get(Set::Player_Fullscreen);

	// maybe we started the player maximized, then showNormal will deliver a strange size
	if( maximized || fullscreen ) {
		target_size = QSize(1024, 800);
	}

	bool is_maximized = this->isMaximized();
	bool is_fullscreen = this->isFullScreen();

	_settings->set(Set::Player_Maximized, is_maximized);
	if(is_maximized) {
		_settings->set(Set::Player_Fullscreen, false);
	}

	// maybe we started the player maximized, then showNormal will deliver a strange size
	if(target_size != this->size() && !is_maximized && !is_fullscreen) {
		resize(target_size);
		return;
	}

	set_album_label();
	lab_artist->setText(Helper::elide_text(_md.artist, lab_artist, 1));
	lab_title->setText(Helper::elide_text(_md.title, lab_title, 2));

	_pph->resize(sz);

	_settings->set(Set::Player_Size, this->size());

	update();
}


void GUI_Player::keyPressEvent(QKeyEvent* e) {

	e->accept();

	switch (e->key()) {

		case (Qt::Key_F9):

		if(!isMaximized())
			this->showMaximized();
		else this->showNormal();
			break;

		case (Qt::Key_F10):
			action_Dark->setChecked(!action_Dark->isChecked());
			break;

		case (Qt::Key_F11):
			show_fullscreen_toggled(!this->isFullScreen());
			break;

		default:
			break;
	}
}

void GUI_Player::showEvent(QShowEvent *e){

	Q_UNUSED(e);

	bool maximized = _settings->get(Set::Player_Maximized);

	if(maximized){
		showMaximized();
	}

	else{
		showNormal();
	}

	_cur_library->resize(library_widget->size());
	ui_playlist->resize(playlist_widget->size());
}



void GUI_Player::closeEvent(QCloseEvent* e) {

	bool min_to_tray = _settings->get(Set::Player_Min2Tray);

	if(min_to_tray) {
		e->ignore();
		this->hide();
	}
}
