/* GUI_Bookmarks.cpp */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "GUI_Bookmarks.h"
#include "Helper/Helper.h"

#define TXT_NO_BOOKMARK "--:--"

GUI_Bookmarks::GUI_Bookmarks(QString name, QWidget *parent) :
	PlayerPlugin(name, parent),
	Ui::GUI_Bookmarks()
{

	setupUi(this);

	_bookmarks = new Bookmarks(this);

	connect(_bookmarks, SIGNAL(sig_bookmarks_changed(const QList<Bookmark>&)),
			this, SLOT(bookmarks_changed(const QList<Bookmark>&)));
	connect(_bookmarks, SIGNAL(sig_next_changed(const Bookmark&,bool)),
			this, SLOT(next_changed(const Bookmark&, bool)));
	connect(_bookmarks, SIGNAL(sig_prev_changed(const Bookmark&,bool)),
			this, SLOT(prev_changed(const Bookmark&, bool)));


	connect(btn_tool, SIGNAL(sig_new()), this, SLOT(new_clicked()));
	connect(btn_tool, SIGNAL(sig_delete()), this, SLOT(del_clicked()));
	connect(btn_bw, SIGNAL(clicked()), this, SLOT(prev_clicked()));
	connect(btn_fw, SIGNAL(clicked()), this, SLOT(next_clicked()));
	connect(cb_loop, SIGNAL(clicked(bool)), this, SLOT(loop_clicked(bool)));
	connect(cb_bookmarks, SIGNAL(currentIndexChanged(int)), this, SLOT(combo_changed(int)));

	btn_tool->show_new(true);
	btn_tool->show_delete(true);

	disable_prev();
	disable_next();

	bookmarks_changed(_bookmarks->get_all_bookmarks());
}

void GUI_Bookmarks::language_changed(){
	retranslateUi(this);
}


void GUI_Bookmarks::bookmarks_changed(const QList<Bookmark>& bookmarks){
	disconnect(cb_bookmarks, SIGNAL(currentIndexChanged(int)), this, SLOT(combo_changed(int)));

	cb_bookmarks->clear();
	for(const Bookmark& bookmark : bookmarks){
		cb_bookmarks->addItem(bookmark.get_name());
	}
	connect(cb_bookmarks, SIGNAL(currentIndexChanged(int)), this, SLOT(combo_changed(int)));
}

void GUI_Bookmarks::disable_prev() {
	btn_bw->setEnabled( false );
	lab_last->setText( TXT_NO_BOOKMARK );
}

void GUI_Bookmarks::disable_next() {
	btn_fw->setEnabled(false);
	lab_next->setText( TXT_NO_BOOKMARK );
}




void GUI_Bookmarks::prev_changed(const Bookmark& bookmark, bool valid){

	if(!valid){
		disable_prev();
		return;
	}

	btn_bw->setEnabled(true);
	lab_last->setText(Helper::cvt_ms_to_string(bookmark.get_time() * 1000, true, true, false));
}

void GUI_Bookmarks::next_changed(const Bookmark& bookmark, bool valid){

	if(!valid){
		disable_next();
		return;
	}

	btn_fw->setEnabled(true);
	lab_next->setText(Helper::cvt_ms_to_string(bookmark.get_time() * 1000, true, true, false));
}


void GUI_Bookmarks::combo_changed(int cur_idx) {

	_bookmarks->jump_to(cur_idx);
}


void GUI_Bookmarks::next_clicked() {
	_bookmarks->jump_next();
}

void GUI_Bookmarks::prev_clicked() {
	_bookmarks->jump_prev();
}

void GUI_Bookmarks::loop_clicked(bool b){
	bool success = _bookmarks->set_loop(b);
	if(!success){
		cb_loop->setChecked(success);
	}
}

void GUI_Bookmarks::new_clicked() {

	if( _bookmarks->is_md_extern() ){
		Message::warning(tr("Sorry, bookmarks can only be set for library tracks at the moment."),
						tr("Bookmarks"));
		return;
	}

	_bookmarks->save();

}


void GUI_Bookmarks::del_clicked() {

	int idx = cb_bookmarks->currentIndex();
	_bookmarks->remove(idx);
}

void GUI_Bookmarks::del_all_clicked() {
	_bookmarks->remove_all();
}
