/* GUI_AbstractStream.h */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef GUI_AbstractStream_H
#define GUI_AbstractStream_H

#include "GUI/MenuTool/MenuTool.h"
#include "GUI/ComboBoxDelegate/ComboBoxDelegate.h"
#include "Interfaces/PlayerPlugin/PlayerPlugin.h"
#include "Components/StreamPlugins/Streams/AbstractStreamHandler.h"
#include "DatabaseAccess/CDatabaseConnector.h"


#include <QComboBox>
#include <QPushButton>
#include <QLineEdit>
#include <QLabel>
#include <QPainter>

class QModelIndex;
class QWidget;
class QVariant;


class GUI_AbstractStream : public PlayerPlugin
{

	Q_OBJECT

private:
	QLineEdit*				_le_url;
	QComboBox*				_combo_stream;
	QPushButton*			_btn_play;
	MenuToolButton*			_btn_tool;
	QLabel*					_lab_listen;

	virtual void			init_connections();
	virtual void			init_streams();

signals:
	void sig_close_event();

public:
	GUI_AbstractStream(const QString& name, AbstractStreamHandler* stream_handler, QWidget* parent=nullptr);
	virtual ~GUI_AbstractStream();


protected:
	CDatabaseConnector*		_db;	// used to fetch entries from DB
	QString					_title_fallback_name;

	AbstractStreamHandler*	_stream_handler;

	// url, name
	QMap<QString, QString>	_stations;


	virtual void						language_changed();
	virtual void						play(QString url, QString station_name);
	virtual GlobalMessage::Answer		show_delete_confirm_dialog();

protected slots:
	virtual void listen_clicked();
	virtual void combo_idx_changed(int idx);
	virtual void combo_text_changed(const QString& text);
	virtual void url_text_changed(const QString& url);
	virtual void delete_clicked();
	virtual void save_clicked();
	virtual void new_clicked();

	void error();
	void data_available();

public:
	virtual void setup_stations(const QMap<QString, QString>&);

	template<typename T>
	void setup_parent(T* subclass){

		subclass->setupUi(subclass);
		_le_url =			subclass->le_url;
		_combo_stream =		subclass->combo_stream;
		_btn_play =			subclass->btn_play;
		_btn_tool =			subclass->btn_tool;
		_lab_listen =		subclass->lab_listen;

		init_connections();
		init_streams();
	}
};

#endif // GUI_AbstractStream_H
