/* Logger.h */

/* Copyright (C) 2011-2015  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifndef LOGGER_H
#define LOGGER_H

#include <ostream>
#include <iostream>
#include <QObject>
#include <QString>
#include <QStringList>

enum class Log : quint8
{
	Warning,
	Error,
	Info,
	Debug
};


class Logger{

protected:
	std::ostream& _out;

public:

	Logger(std::ostream& out) :
		_out(out)
	{

	}

	Logger(const char* msg, std::ostream& out) :
		Logger(out)
	{
		_out << msg;


	}

	template <typename T>
	Logger& operator << (const T& msg){
		_out << msg;
		return *this;
	}

	Logger& operator << (const QString& msg){
		(*this) << msg.toLocal8Bit();
		return *this;
	}

	Logger& operator << (const QStringList& lst){
		for(const QString& str : lst){
			(*this) << str << ", ";
		}

		return *this;
	}

	Logger& operator << (const QByteArray& arr){
		for(const QChar& c : arr){
			(*this) << c;
		}

		return *this;
	}

	Logger& operator << (const QChar& c){

		(*this) << c.toLatin1();


		return *this;
	}


	template<typename T, template <typename ELEM> class CONT>
	Logger& operator << (const CONT<T> list){

		for(const T& item : list){
			(*this) << item << ", ";
		}


		return *this;
	}


	~Logger(){
		_out << std::endl;
		_out.flush();
	}

};

Logger sp_log(Log type=Log::Debug, const QString& module=QString());
#endif // LOGGER_H
