/* LibraryPluginLoader.cpp */

/* Copyright (C) 2011-2014  Lucio Carreras
 *
 * This file is part of sayonara player
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */



#include "LibraryPluginLoader.h"
#include "LocalLibraryInterface/LocalLibraryContainer.h"

LibraryPluginLoader::LibraryPluginLoader(QObject* parent) :
	QObject(parent),
	SayonaraClass()
{

	QString lib_dir = Helper::getLibPath();

	QMap<QString, QIcon> lib_chooser_labels;
	QString preferred_plugin("Local Library");


	LocalLibraryContainer* local_library = new LocalLibraryContainer(this);
	_libraries << local_library;
	lib_chooser_labels[local_library->get_display_name()] = local_library->get_icon();

	_cur_idx = 0;

	QDir plugin_dir = QDir(lib_dir);
	QStringList entry_list = plugin_dir.entryList(QDir::Files);

	for(int i=0; i<entry_list.size(); i++) {

		QString filename = entry_list[i];
		QPluginLoader loader(plugin_dir.absoluteFilePath(filename));

		QObject* plugin = loader.instance();
		if(!plugin) continue;

		LibraryContainer* library_container = dynamic_cast<LibraryContainer*>(plugin);
		if(!library_container) continue;

		sp_log(Log::Info) << "Found library plugin " << library_container->get_name();
		_libraries << library_container;

		QString display_name = library_container->get_display_name();
		QIcon icon = library_container->get_icon();

		lib_chooser_labels[display_name] = icon;

		if(!preferred_plugin.compare(library_container->get_name(), Qt::CaseInsensitive)){
			_cur_idx = _libraries.size() -1;
		}
	}

	sp_log(Log::Info) << "Found " << _libraries.size() << " library types";

	int idx=0;

	for(LibraryContainer* container : _libraries ){
		container->get_ui()->set_lib_chooser(lib_chooser_labels);
		container->get_ui()->set_index(idx++);
	}
}

LibraryPluginLoader::~LibraryPluginLoader()
{

}


QList<LibraryContainer*> LibraryPluginLoader::get_libraries() const {
	return _libraries;
}

QList<AbstractLibrary*> LibraryPluginLoader::get_libraries_logic() const {
	QList<AbstractLibrary*> libraries;
	for(const LibraryContainer* container : _libraries){
		libraries << container->get_logic();
	}

	return libraries;
}

QList<GUI_AbstractLibrary*> LibraryPluginLoader::get_libraries_ui() const {
	QList<GUI_AbstractLibrary*> libraries;
	for(const LibraryContainer* container : _libraries){
		libraries << container->get_ui();
	}

	return libraries;
}


AbstractLibrary* LibraryPluginLoader::get_cur_library_logic() const {

	if(_cur_idx >= _libraries.size() || _cur_idx < 0) {
		return nullptr;
	}

	return _libraries[_cur_idx]->get_logic();
}

GUI_AbstractLibrary* LibraryPluginLoader::get_cur_library_ui() const {

	if(_cur_idx >= _libraries.size() || _cur_idx < 0) {
		return nullptr;
	}

	return _libraries[_cur_idx]->get_ui();
}


